@@ -7,6 +7,8 @@
 #ifndef INCLUDE_revwalk_h__
 #define INCLUDE_revwalk_h__
 
+#include "common.h"
+
 #include "git2/revwalk.h"
 #include "oidmap.h"
 #include "commit_list.h"
@@ -14,7 +16,7 @@
 #include "pool.h"
 #include "vector.h"
 
-GIT__USE_OIDMAP;
+#include "oidmap.h"
 
 struct git_revwalk {
 	git_repository *repo;
